/*
 *	Tykkipeli 2
 * Various info windows are gathered in this module.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include "defines.h"
#include "tp2.h"
#include "game_screen.h"
#include "players.h"
#include "misc.h"
#include "option_screen.h"
#include "option_screen_defines.h"
#include "weapons.h"
#include "shields.h"
#include "terrain.h"
#include "change_item.h"
#include "colors.h"

#include "hr_colors.h"
#include "hr_windows.h"
#include "hr_defines.h"
#include "hr_fonts.h"
#include "hr_mouse.h"
#include "hr_buttons.h"
#include "hr_bobs.h"
#include "hr_general.h"
#include "hr_input_handler.h"
#include "hr_ppm.h"
#include "hr_ilbm.h"
#include "hr_keyboard.h"
#include "hr_screens.h"
#include "hr_zones.h"
#include "hr_graphics.h"
#include "hr_strings.h"


/* Opens a window on mouse pointer. Used on game screen */
void info_window(int x,int y)
{
	struct player *p=firstplayer;
	struct hr_window *wp;
	int xsize,ysize,xoffset;
	/* Check whether it was a player that was clicked on*/
	while(p!=NULL) {
		if(DISTANCE(x,y,p->x,p->y)<10)
		  break;
		p=p->next;
	}
	/* Select window size.*/
	if(p!=NULL) {
		/* Player info */
		xsize=230;
		ysize=94;
	}
	else {
		/* Terrain info */
		xsize=210;
		if(game_flags & TERRAIN_PIXMAPS)
		  ysize=42;
		else
		  ysize=55;
	}
	/* Calc coordinates of window's topleft corner */
	x=MAX(x-xsize/2,1);
	x=MIN(x,MAX_X-xsize);
	y=MAX(y-ysize/2,1);
	y=MIN(y,MAX_Y-ysize);
	/* Open the window. No gadgets.*/
	wp=hr_window_open(x,y,xsize,ysize,hr.def_colors,NONE);
	/* From now on xsize is the width of window contents
	 * and x the x coordinate of value gadgets.*/
	xsize-=6;
	x+=4;
	/* Player info */
	if(p!=NULL) {
		xoffset=98;
		hr_set_color_index(temp_colors,COLOR_FOREGROUND,p->britecolor);
		hr_textbox(x-2,y+2,x+xsize-1,y+13,
					  hr.font8x8,
					  temp_colors,p->name,
					  STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		hr_value_gadget(x,y+15,xsize,12,xoffset,
							 hr.font8x8,hr.def_colors,
							 "Wealth",NULL,p->wealth,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		hr_value_gadget(x,y+28,xsize,12,xoffset,
							 hr.font8x8,hr.def_colors,
							 "Rounds won",NULL,p->rounds_won,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		hr_value_gadget(x,y+41,xsize,12,xoffset,
							 hr.font8x8,hr.def_colors,
							 "Deaths",NULL,p->deaths,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		hr_value_gadget(x,y+54,xsize,12,xoffset,
							 hr.font8x8,hr.def_colors,
							 "Kills",NULL,p->kills,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		hr_value_gadget(x,y+67,xsize,12,xoffset,
							 hr.font8x8,hr.def_colors,
							 "Suicides",NULL,p->suicides,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		hr_value_gadget(x,y+80,xsize,12,xoffset,
							 hr.font8x8,hr.def_colors,
							 "Magic ratio",NULL,
							 magic_ratio(p->deaths,p->kills,p->suicides),
							 BUTTON_PRESSED|BUTTON_ENABLED);
	}
	/* Game info */
	else {
		xoffset=80;
		hr_textbox(x-2,y+2,x+xsize-1,y+13,
					  hr.font8x8,
					  hr.def_colors,"Terrain info",
					  STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
      hr_value_gadget(x,y+15,xsize,12,xoffset,
							 hr.font8x8,hr.def_colors,
							 "Gravity",gravity_string(buffer),0,
							 BUTTON_PRESSED|BUTTON_ENABLED);
      hr_value_gadget(x,y+28,xsize,12,xoffset,
							 hr.font8x8,hr.def_colors,
							 "Wind",windtext[windmode],0,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		/* Do not print steepness info if terrain pixmaps are in use.*/
		if(!(game_flags & TERRAIN_PIXMAPS))
		  hr_value_gadget(x,y+41,xsize,12,xoffset,
								hr.font8x8,hr.def_colors,
								"Steepness",
								hr_percentage_string(buffer,
															0,MAX_BUMPINESS,DEF_BUMPINESS,
															bumpiness,
															"Min.","Max.","Normal"),0,
								BUTTON_PRESSED|BUTTON_ENABLED);
	}
	hr_input_handler(NULL,NULL,AMB_RELEASED);
	hr_window_close(wp);
}


/* Roster info window for option screen.
 * This function prints text in file "tp2_datapath/<name_of_player>.info".*/
void option_rosterinfo(int r)
{
	FILE *fp;
	struct hr_window *w;	
	int y=0;
	/* Open a window.*/
	w=hr_window_open(50,50,540,380,hr.def_colors,WINDOW_CLIPPING);
	/* Create gadgets */
	w->gadgets=hr_create_window_gadgets(1);
	hr.default_button=(w->gadgets->button)[0]=
	  hr_create_button(0,0,0,0,
							 hr.font8x8,hr.def_colors,
							 "Ok","Bye",0,BUTTON_ENABLED|BUTTON_INVISIBLE);
	hr_window_refresh(w);
	/* Rest of the contents.*/
	hr_textbox(52,52,587,63,
				  hr.font8x8,
				  hr.def_colors,roster[r].name,
				  STRING_START,STRING_END,
				  BUTTON_PRESSED|BUTTON_ENABLED);
	/* Create file name */
	sprintf(buffer,"%s/players/roster%d.info",tp2_datapath,r);
	/* Open file and print it */
	if((fp=fopen(buffer,"r"))) {
		while(fgets(buffer,65,fp))
		  HR_CENTER(320,70+(y++)*10,2,0,buffer,hr.font8x8,NONE);
		fclose(fp);
	}
	while(hr_input_handler(hr_input_handler(NULL,NULL,LMB_PRESSED),
								  NULL,LMB_RELEASED)!=(w->gadgets->button)[0]);
	hr_window_close(w);
}


/* Prints quick info about a players statistics.
 * Used on option screen.*/
int option_quickinfo(int r)
{
	FILE *fp;
	struct hr_window *w;
	struct hr_button *ob,*b,*b1,*b2,*b3;
	int x,y,xoffset=100,xsize=300,ysize=136,bflags;
	int name_changed=0;
	/* Calc window position */
	x=330;
	y=MIN(470-ysize,10+r*14-ysize/2);
	y=MAX(y,10);
	/* Open window.*/
	w=hr_window_open(x,y,xsize,ysize,hr.def_colors,WINDOW_CLIPPING);
	/* Create gadgets */
	w->gadgets=hr_create_window_gadgets(2);
	/* B1 */
	hr.default_button=(w->gadgets->button)[0]=
	  hr_create_button(0,0,0,0,
							 hr.font8x8,hr.def_colors,
							 "~uO~uk",NULL,'o',
							 BUTTON_ENABLED|BUTTON_INVISIBLE);
	/* B2*/
	(w->gadgets->button)[1]=
	  hr_create_button(0,0,0,0,
							 hr.font8x8,hr.def_colors,
							 "More player ~ui~unfo",NULL,'i',
							 BUTTON_ENABLED|BUTTON_INVISIBLE);
	hr_window_refresh(w);
	/* Disable "more info"-button if there is no
	 * player data file in tp2_datapath */
	sprintf(buffer,"%s/players/roster%d.info",tp2_datapath,r);
	if((fp=fopen(buffer,"r")))
	  fclose(fp);
	else{
		bflags=NONE;
		hr_update_button((w->gadgets->button)[1],UPDATE_FLAGS,&bflags);
	}
	/* Name. Can be changed.*/
   b1=hr_create_button(x+2,y+2,x+xsize-3,y+13,
							  hr.font8x8,hr.def_colors,
							  NULL,roster[r].name,0,
							  BUTTON_PRESSED|BUTTON_ENABLED|BUTTON_ACCEPTS_INPUT);
	/* xsize == width of window contents. */
	xsize-=6;
	x+=4;
	/* Print info*/
	hr_value_gadget(x,y+15,xsize,12,xoffset,
						 hr.font8x8,hr.def_colors,
						 "Games",NULL,roster[r].games,
						 BUTTON_PRESSED|BUTTON_ENABLED);
	hr_value_gadget(x,y+28,xsize,12,xoffset,
						 hr.font8x8,hr.def_colors,
						 "Games won",NULL,roster[r].won,
						 BUTTON_PRESSED|BUTTON_ENABLED);
	hr_value_gadget(x,y+41,xsize,12,xoffset,
						 hr.font8x8,hr.def_colors,
						 "Deaths",NULL,roster[r].deaths,
						 BUTTON_PRESSED|BUTTON_ENABLED);
	hr_value_gadget(x,y+54,xsize,12,xoffset,
						 hr.font8x8,hr.def_colors,
						 "Kills",NULL,roster[r].kills,
						 BUTTON_PRESSED|BUTTON_ENABLED);
	hr_value_gadget(x,y+67,xsize,12,xoffset,
						 hr.font8x8,hr.def_colors,
						 "Suicides",NULL,roster[r].suicides,
						 BUTTON_PRESSED|BUTTON_ENABLED);
	hr_value_gadget(x,y+80,xsize,12,xoffset,
						 hr.font8x8,hr.def_colors,
						 "Magic ratio",NULL,magic_ratio(roster[r].deaths,
																  roster[r].kills,
																  roster[r].suicides),
						 BUTTON_PRESSED|BUTTON_ENABLED);
	/* Weapon */
	b2=hr_create_button(x,y+93,x+xoffset-2,y+104,
							  hr.font8x8,hr.def_colors,
							  "~uW~ueapon",NULL,'w',BUTTON_ENABLED);
	hr_textbox(x+xoffset,y+93,x+xsize-1,y+104,
				  hr.font8x8,hr.def_colors,
				  get_weapon(roster[r].default_weapon)->name,
				  STRING_START,STRING_END,
				  BUTTON_PRESSED|BUTTON_ENABLED);
	/* Shield*/
	b3=hr_create_button(x,y+106,x+xoffset-2,y+117,
							  hr.font8x8,hr.def_colors,
							  "~uS~uhield",NULL,'s',BUTTON_ENABLED);
	hr_textbox(x+xoffset,y+106,x+xsize-1,y+117,
				  hr.font8x8,hr.def_colors,
				  get_shield(roster[r].default_shield)->name,
				  STRING_START,STRING_END,
				  BUTTON_PRESSED|BUTTON_ENABLED);
	
	/* Wait until "Ok" is clicked, then close window.
	 * The player's name can be edited.*/
	b=NULL;
	while(b!=(w->gadgets->button)[0]) {
		ob=hr_input_handler(NULL,&options_effect,LMB_PRESSED);
		b=hr_input_handler(ob,&options_effect,LMB_RELEASED);
		if(b!=NULL){
			if(b==(w->gadgets->button[1])){
				/* "More info" clicked? */
				option_rosterinfo(r);
			}
			else if(b==b2){
				/* Change def weapon */
				change_default_item(r,0);
				hr_textbox(x+xoffset,y+93,x+xsize-1,y+104,
							  hr.font8x8,hr.def_colors,
							  get_weapon(roster[r].default_weapon)->name,
							  STRING_START,STRING_END,
							  BUTTON_PRESSED|BUTTON_ENABLED);
			}
			else if(b==b3){
				/* Change def shield */
				change_default_item(r,1);
				hr_textbox(x+xoffset,y+106,x+xsize-1,y+117,
							  hr.font8x8,hr.def_colors,
							  get_shield(roster[r].default_shield)->name,
							  STRING_START,STRING_END,
							  BUTTON_PRESSED|BUTTON_ENABLED);
			}
		}
	}
	/* Update buttons. Ugly - FIX.*/
	if(b1->flags & BUTTON_STRING_EDITED){
		b1->flags &= ~(BUTTON_STRING_EDITED);
		/* Update roster name data.*/
		free(roster[r].name);
		roster[r].name=malloc(strlen(b1->text2)+1);
		strcpy(roster[r].name,b1->text2);
		name_changed=1;
	}
	hr_delete_button(b1);   /* Name */
	hr_delete_button(b2);   /* Weapon */
	hr_delete_button(b3);   /* Shield */
	/* Finally...*/
	hr_window_close(w);
	return name_changed;
}


/* Weptogram(tm) */
void weptogram(struct weapon *wp,int x1,int y1,int x2,int y2,int flags)
{
	struct warhead_list *wl;
	int xx,yy,zz,cx1,cy1,cx2,cy2;
	cx1=__clipx1;
	cy1=__clipy1;
	cx2=__clipx2;
	cy2=__clipy2;
	gl_setclippingwindow(x1,y1,x2,y2);
	hr_bevelbox(x1,y1,x2,y2,hr.def_colors,flags);
	for(yy=0;yy<(wp->shots);yy++){
		xx=0;
		wl=(wp->warheads)[yy];
		zz=warheads_in_list(wl);
		while(wl!=NULL){
			gl_setpixel(x1+(x2-x1)/2-(zz/2)+xx,
							y1+1+abs((wp->speed)[yy])/10,
							MINCOL_BULLET+(wl->type->color));
			++xx;
			wl=wl->next;
		}
	}
	gl_setclippingwindow(cx1,cy1,cx2,cy2);
}


/* Weapon/Shield Information for shop screen.*/
void item_information(int m,void *ptr,int x1,int y1,int x2,int y2)
{
	FILE *fp;
	struct hr_window *win;
	struct weapon *wp;
	struct shield *sp;
	int yoffset,y;
	int xx,yy,w,h;
	/* Create window */
	w=(x2-x1+1);
	h=(y2-y1+1);
	win=hr_window_open(x1,y1,w,h,hr.def_colors,NONE);
	/* Create gadgets */
	win->gadgets=hr_create_window_gadgets(1);
	hr.default_button=(win->gadgets->button)[0]=
	  hr_create_button(0,0,0,0,
							 hr.font8x8,hr.def_colors,
							 "How very interesting!",NULL,0,
							 BUTTON_ENABLED|BUTTON_INVISIBLE);
	hr_window_refresh(win);
	/* Weapon info */
#define X_OFFSET 200
	if(m==0) {
		wp=(struct weapon *)(ptr);
		/* Name */
		hr_textbox((win->zone->x1)+2,(win->zone->y1)+2,
					  (win->zone->x2)-2,(win->zone->y1)+13,
					  hr.font8x8,hr.def_colors,wp->name,
					  STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		/* Cost */
		hr_value_gadget((win->zone->x1)+50,(win->zone->y1)+17,w-100,12,X_OFFSET,
							 hr.font8x8,hr.def_colors,"Cost",NULL,wp->cost,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		/* Amount of warheads */
		xx=0;
		for(yy=0;yy<(wp->shots);yy++)
		  xx+=warheads_in_list((wp->warheads)[yy]);
      hr_value_gadget((win->zone->x1)+50,(win->zone->y1)+30,w-100,12,X_OFFSET,
							 hr.font8x8,hr.def_colors,"Initial warheads",NULL,xx,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		/* Shots */
		hr_value_gadget((win->zone->x1)+50,(win->zone->y1)+43,w-100,12,X_OFFSET,
							 hr.font8x8,hr.def_colors,"Shots",NULL,wp->shots,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		/* Other text, if it is available. */
		sprintf(buffer,"%s/weapons/%s.info",tp2_datapath,wp->name);
		yoffset=(win->zone->y1)+56;
#if 1
		weptogram(wp,(win->zone->x2)-48,(win->zone->y1)+17,
					 (win->zone->x2)-2,yoffset-2,BUTTON_PRESSED);
#endif
	}
	/* Shield info */
	else {
		sp=(struct shield *)(ptr);
		/**/
		hr_textbox((win->zone->x1)+2,(win->zone->y1)+2,
					  (win->zone->x2)-2,(win->zone->y1)+13,
					  hr.font8x8,hr.def_colors,sp->name,
					  STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		hr_value_gadget((win->zone->x1)+50,(win->zone->y1)+17,w-100,12,X_OFFSET,
							 hr.font8x8,hr.def_colors,"Cost",NULL,sp->cost,
							 BUTTON_PRESSED|BUTTON_ENABLED);
      hr_value_gadget((win->zone->x1)+50,(win->zone->y1)+30,w-100,12,X_OFFSET,
							 hr.font8x8,hr.def_colors,"Range",NULL,sp->range,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		hr_value_gadget((win->zone->x1)+50,(win->zone->y1)+43,w-100,12,X_OFFSET,
							 hr.font8x8,hr.def_colors,"Efficiency",NULL,sp->efficiency,
							 BUTTON_PRESSED|BUTTON_ENABLED);
		/* Other text, if it is available. */
		sprintf(buffer,"%s/shields/%s.info",tp2_datapath,sp->name);
		yoffset=(win->zone->y1)+58;
	}
	/* Print other text, if available */
	hr_bevelbox((win->zone->x1)+4,yoffset,
					(win->zone->x2)-4,(win->zone->y2)-15,
					hr.def_colors,BUTTON_PRESSED);
	y=0;
	hr_replace_chars(buffer," ","_");
	if((fp=fopen(buffer,"r"))) {
		while(fgets(buffer,65,fp))
		  HR_CENTER(320,yoffset+2+((y++))*10,2,0,buffer,hr.font8x8,NONE);
		fclose(fp);
	}
	else
	  HR_CENTER(320,(412+yoffset)/2-4,2,0,
					"No other info available.",hr.font8x8,NONE);
	while(!(hr_input_handler(hr_input_handler(NULL,NULL,LMB_PRESSED),
									 	NULL,LMB_RELEASED)));
	hr_window_close(win);
}


/* Called after round */
void post_round_info(struct player *winner)
{
	struct hr_window *win;
	struct hr_button *b,*ob,*tb;
	struct player *p;
	unsigned char *buf2;
	int x,c,closewin=0,bflags;
#define DEATH_COMMENTS 12
	char *deathcomment[]=  {
		"Dead","DooMed","Wasted","Stackered","Trashed","M$ized","Cleric",
		  "Massacred","Vaporized","Powderized","Disintegrated","Disassembled"
	};
	/* Post-round info window*/
	while(!closewin) {
		win=hr_window_open(40,40,S_WIDTH-80,S_HEIGHT-80,
								 hr.def_colors,NONE);
		win->gadgets=hr_create_window_gadgets(2);
		b=hr.default_button=(win->gadgets->button)[0]=
		  hr_create_button(0,0,0,0,
								 hr.font8x8,hr.def_colors,
								 ((round<round_n-1)?
								  ("Go to next round"):("See Winner")),
								 NULL,0,BUTTON_ENABLED|BUTTON_INVISIBLE);
		ob=(win->gadgets->button)[1]=
		  hr_create_button(0,0,0,0,
								 hr.font8x8,hr.def_colors,
								 "Admire scenery some more",NULL,0,
								 BUTTON_ENABLED|BUTTON_INVISIBLE);
		hr_window_refresh(win);
		/* Fill the window with some nice & useless info 8-) */
		if(round==(round_n-1))
		  sprintf(buffer,"Final r");
		else
		  sprintf(buffer,"R");
		buf2=malloc(100);
		sprintf(buf2,"esults after %d round",round+1);
		strcat(buffer,buf2);
		free(buf2);
		if(round>0)
		  strcat(buffer,"s:");
		else
		  strcat(buffer,":");
		HR_CENTER(WIDTH/2,50,2,0,buffer,hr.font8x8,NONE);
		
		HR_CENTER(WIDTH/2,70,2,0,"MADNESS!",hr.font8x8,NONE);
		sprintf(buffer,"Within %d turn",turn);
		if(turn>1)
		  strcat(buffer,"s");
		HR_CENTER(WIDTH/2,90,2,0,buffer,hr.font8x8,NONE);
		
		if(shots_fired>1)
		  sprintf(buffer,"%d shots were fired.",shots_fired);
		else
		  sprintf(buffer,"One shot was fired.");
		HR_CENTER(WIDTH/2,110,2,0,buffer,hr.font8x8,NONE);
		for(x=0;x<S_WIDTH;x++)
		  land_total+=terrain[x];
		sprintf(buffer,"%ld cubic meters of earth was destroyed.",land_total);
		HR_CENTER(WIDTH/2,120,2,0,buffer,hr.font8x8,NONE);
		
		sprintf(buffer,"%d credits worth of ammo was used on purpose.",
				  ammo_money);
		HR_CENTER(WIDTH/2,130,2,0,buffer,hr.font8x8,NONE);
		
		/* X-offset of window's right edge from right edge of whole screen.*/
#define X_OFF (S_WIDTH-54)
		
		/* Player info boxes and stuff */
		hr_bevelbox(50,300,S_WIDTH-50,S_HEIGHT-60,hr.def_colors,BUTTON_PRESSED);
		hr_bevelbox(54,304,S_WIDTH-54,315,hr.def_colors,BUTTON_PRESSED);/*xyxxy*/
		
		hr_write(56,306,1,0,"Player",hr.font8x8,STRING_START,STRING_END,NONE);
		hr_write(X_OFF-238,306,1,0,"$Earned",
					hr.font8x8,STRING_START,STRING_END,NONE);
		hr_write(X_OFF-168,306,1,0,"$Total",
					hr.font8x8,STRING_START,STRING_END,NONE);
		hr_write(X_OFF-98,306,1,0,"Status",
					hr.font8x8,STRING_START,STRING_END,NONE);
		
		/* Print player names */
		p=firstplayer;c=0;
		while(p!=NULL) {
			if(p==winner)
			  gl_fillbox(55,318+c*10,S_WIDTH-109,10,p->firstcolor);
			strcpy(buffer,p->name);
			/*  Cut away the part of name that does not fit to 1st box.*/
			buffer[(S_WIDTH-290)/8]=0; /* Bug for BIG resolutions :-( ;-) */
			hr_write(56,319+c*10,2,0,buffer,
						hr.font8x8,STRING_START,STRING_END,NONE);
			sprintf(buffer,"%d",((p->wealth)-(p->old_wealth)));
			hr_write(X_OFF-174-strlen(buffer)*8,319+c*10,2,0,
							 buffer,hr.font8x8,STRING_START,STRING_END,NONE);
			sprintf(buffer,"%d",p->wealth);
			hr_write(X_OFF-104-strlen(buffer)*8,319+c*10,2,0,
							 buffer,hr.font8x8,STRING_START,STRING_END,NONE);
			if((p->flags) & ALIVE)
			  sprintf(buffer,"%s","Alive!");
			else
			  sprintf(buffer,deathcomment[random()%DEATH_COMMENTS]);
			hr_write(X_OFF-97,319+c*10,2,0,buffer,
						hr.font8x8,STRING_START,STRING_END,NONE);
			c++;
			p=p->next;
		}
		
		/* Now draw boxes around player info... */
		hr_bevelbox(54,317,X_OFF-242,S_HEIGHT-64,hr.def_colors,BUTTON_PRESSED);
		hr_bevelbox(X_OFF-240,317,X_OFF-172,S_HEIGHT-64,hr.def_colors,BUTTON_PRESSED);
		hr_bevelbox(X_OFF-170,317,X_OFF-102,S_HEIGHT-64,hr.def_colors,BUTTON_PRESSED);
		hr_bevelbox(X_OFF-100,317,X_OFF,S_HEIGHT-64,hr.def_colors,BUTTON_PRESSED);
		gl_line(X_OFF-241,317,X_OFF-241,S_HEIGHT-64,0);
		gl_line(X_OFF-171,317,X_OFF-171,S_HEIGHT-64,0);
		gl_line(X_OFF-101,317,X_OFF-101,S_HEIGHT-64,0);
		/* Loop until either button is clicked.*/
		tb=NULL;
		while(tb==NULL){
			tb=hr_input_handler(NULL,NULL,LMB_PRESSED);
			tb=hr_input_handler(tb,NULL,LMB_RELEASED);
		}
		
		/* Mark window to be closed for good if 1st button was selected.*/
		if(tb==b)
		  closewin=1;
		/* Bye bye window.*/
		hr_window_close(win);
		/* "Admire scenery"-mode... */
		if(!closewin) {
			ob=hr_create_button(0,PANEL_Y+1,MAX_X,PANEL_Y+12,
									  hr.font8x8,hr.def_colors,
									  "Plenty of scorched earth there... Click here to continue",
									  NULL,0,BUTTON_ENABLED|BUTTON_INVISIBLE);
			hr.default_button=ob;
			bflags=BUTTON_ENABLED;
			hr_update_button(ob,UPDATE_FLAGS,&bflags);
			b=NULL;
			while(b!=ob) {
				b=hr_input_handler(NULL,NULL,AMB_PRESSED);
				if(HR_RMB) {
					if(hr_mouse.y<=MAX_Y)
					  info_window(hr_mouse.x,hr_mouse.y);
					b=NULL;
				}
				else
				  b=hr_input_handler(b,NULL,LMB_RELEASED|RMB_RELEASED);
			}
			/* Ugly but works.*/
			hr_shade(ob->zone->x1,ob->zone->y1,
						ob->zone->x2,ob->zone->y2,hr.def_colors);
			hr_delete_button(ob);
		}
	}
}
	

/* Grabs screen and saves it to /tmp/dadasdsadsadasfsd.*/
int grabzen()
{
	unsigned char tmp[BUFFER_SIZE];
	sprintf(tmp,"/tmp/tp2dump_%ld.ppm",
			  (long unsigned int)(random()%100000));
	printf("Screen grabbed to \"%s\".\n",tmp);
	return hr_save_ppm(0,0,WIDTH,HEIGHT,tmp,NONE);
}


/* Control panel help window that lets one edit colors and remove tracers.*/
int control_colors()
{
	int ok=0,w;
	struct hr_window *win;
	struct hr_button *b,*b1,*b2,*b3;
	/* Open window */
	w=WIDTH/3;
	win=hr_window_open(WIDTH/2-w/2,HEIGHT/2-50,w,100,
							 hr.def_colors,NONE);
	/* Gadgets */
	win->gadgets=hr_create_window_gadgets(1);
	hr.default_button=(win->gadgets->button)[0]=
	  hr_create_button(0,0,0,0,
							 hr.font8x8,hr.def_colors,
							 "~uO~uk",NULL,'o',BUTTON_ENABLED|BUTTON_INVISIBLE);
	hr_window_refresh(win);
	/* 3 selections...*/
	b1=hr_create_button((win->zone->x1)+2,(win->zone->y1)+2,
							  (win->zone->x1)+w-3,(win->zone->y1)+15,
							  hr.font8x8,hr.def_colors,
							  "Recolor ~us~uky",NULL,'s',
							  ((game_flags & SKY_PIXMAPS)?
								(NONE):(BUTTON_ENABLED)));
	b2=hr_create_button((win->zone->x1)+2,(win->zone->y1)+17,
							  (win->zone->x1)+w-3,(win->zone->y1)+30,
							  hr.font8x8,hr.def_colors,
							  "Recolor ~ug~uround",NULL,'g',
							  ((game_flags & TERRAIN_PIXMAPS)?
								(NONE):(BUTTON_ENABLED)));
	b3=hr_create_button((win->zone->x1)+2,(win->zone->y1)+32,
							  (win->zone->x1)+w-3,(win->zone->y1)+45,
							  hr.font8x8,hr.def_colors,
							  "Remove ~ut~uracers",NULL,'t',
							  (((game_flags & TRACERS) && (turn>1))?
								(BUTTON_ENABLED):(NONE)));
	/* Wait for button selection*/
	b=NULL;
	while(!ok){
		b=hr_input_handler(hr_input_handler(NULL,NULL,
														LMB_PRESSED|KBD_PRESSED),
								 NULL,AMB_RELEASED);
		if(b!=NULL){
			/* What button was it? */
			if(b==(win->gadgets->button)[0])
			  ok=1;
			else if(b==b1)
			  set_sky_palette();
			else if(b==b2)
			  set_ground_palette();
			else if(b==b3)
			  ok=2;
		}
	}
	/* Close window and free stuff */
	hr_delete_button(b1);
	hr_delete_button(b2);
	hr_delete_button(b3);
	hr_window_close(win);
	/* This can only be done after window is closed.*/
	if(ok==2)
	  remove_tracers();
	return ok;
}
